@@ -9,14 +9,26 @@
 #ifndef XBLAS_UTILS_HPP
 #define XBLAS_UTILS_HPP
 
+#include <stdexcept>
+#include <tuple>
+#include <type_traits>
+
 #include "xflens/cxxblas/typedefs.h"
 #include "xtensor/xutils.hpp"
 
+#ifndef DEFAULT_LEADING_STRIDE_BEHAVIOR
+#define DEFAULT_LEADING_STRIDE_BEHAVIOR throw std::runtime_error("No valid layout chosen.");
+#endif
+
+#ifndef DEFAULT_STORAGE_ORDER_BEHAVIOR
+#define DEFAULT_STORAGE_ORDER_BEHAVIOR throw std::runtime_error("Cannot handle layout_type of e.");
+#endif
+
 namespace xt
 {
     template <layout_type L = layout_type::row_major, class T>
     inline auto view_eval(T&& t)
-        -> std::enable_if_t<has_raw_data_interface<T>::value && std::decay_t<T>::static_layout == L, T&&>
+        -> std::enable_if_t<has_raw_data_interface<std::decay_t<T>>::value && std::decay_t<T>::static_layout == L, T&&>
     {
         return std::forward<T>(t);
     }
@@ -31,7 +43,7 @@ namespace xt
 
     template <layout_type L = layout_type::row_major, class T, class I = std::decay_t<T>>
     inline auto view_eval(T&& t)
-        -> std::enable_if_t<(!has_raw_data_interface<T>::value || I::static_layout != L)
+        -> std::enable_if_t<(!has_raw_data_interface<std::decay_t<T>>::value || I::static_layout != L)
                             && detail::is_array<typename I::shape_type>::value,
                             xtensor<typename I::value_type,
                                     std::tuple_size<typename I::shape_type>::value,
@@ -42,7 +54,7 @@ namespace xt
 
     template <layout_type L = layout_type::row_major, class T, class I = std::decay_t<T>>
     inline auto view_eval(T&& t)
-        -> std::enable_if_t<(!has_raw_data_interface<T>::value || I::static_layout != L) &&
+        -> std::enable_if_t<(!has_raw_data_interface<std::decay_t<T>>::value || I::static_layout != L) &&
                             !detail::is_array<typename I::shape_type>::value,
                             xarray<typename I::value_type, detail::layout_remove_any(L)>>
     {
@@ -83,7 +95,7 @@ namespace xt
         {
             return cxxblas::StorageOrder::ColMajor;
         }
-        throw std::runtime_error("Cannot handle layout_type of e.");
+        DEFAULT_STORAGE_ORDER_BEHAVIOR;
     }
 
     /**
@@ -93,13 +105,13 @@ namespace xt
     template <class A, std::enable_if_t<A::static_layout == layout_type::row_major>* = nullptr>
     inline BLAS_IDX get_leading_stride(const A& a)
     {
-        return (BLAS_IDX) (a.strides().front() == 0 ? a.shape().back() : a.strides().front());
+        return static_cast<BLAS_IDX>(a.strides().front() == 0 ? a.shape().back() : a.strides().front());
     }
 
     template <class A, std::enable_if_t<A::static_layout == layout_type::column_major>* = nullptr>
     inline BLAS_IDX get_leading_stride(const A& a)
     {
-        return (BLAS_IDX) (a.strides().back() == 0 ? a.shape().front() : a.strides().back());
+        return static_cast<BLAS_IDX>(a.strides().back() == 0 ? a.shape().front() : a.strides().back());
     }
 
     template <class A, std::enable_if_t<A::static_layout != layout_type::row_major && A::static_layout != layout_type::column_major>* = nullptr>
@@ -107,16 +119,13 @@ namespace xt
     {
         if (a.layout() == layout_type::row_major)
         {
-            return (BLAS_IDX) (a.strides().front() == 0 ? a.shape().back() : a.strides().front());
+            return static_cast<BLAS_IDX>(a.strides().front() == 0 ? a.shape().back() : a.strides().front());
         }
         else if (a.layout() == layout_type::column_major)
         {
-            return (BLAS_IDX) (a.strides().back() == 0 ? a.shape().front() : a.strides().back());
-        }
-        else
-        {
-            throw std::runtime_error("No valid layout chosen.");
+            return static_cast<BLAS_IDX>(a.strides().back() == 0 ? a.shape().front() : a.strides().back());
         }
+        DEFAULT_LEADING_STRIDE_BEHAVIOR;
     }
 
     /*******************************
